import logging

import pandas
from dbrepo.RestClient import RestClient
import time

from api.VAMAS import VAMASExperiment, VAMASBlock

logger = logging.getLogger(__name__)
logging.basicConfig(format='%(levelname)s\t%(message)s', filename='/dev/stdout', encoding='utf-8', level=logging.DEBUG)


class DBRepoClient:

    def __init__(self, endpoint: str, username: str, password: str):
        self.client = RestClient(endpoint=endpoint, username=username, password=password)

    def insert_experiment_data(self, database_id: int, table_id, data: VAMASExperiment) -> int:
        payload = dict({
            "format_identifier": data.format_identifier,
            "institution_identifier": data.institution_identifier,
            "instrument_model_identifier": data.instrument_model_identifier,
            "operator_identifier": data.operator_identifier,
            "experiment_identifier": data.experiment_identifier,
            "number_of_lines_in_comment": data.number_of_lines_in_comment,
            "comment": data.comment,
            "experiment_mode": data.experiment_mode,
            "scan_mode": data.scan_mode,
            "number_of_spectral_regions": data.number_of_spectral_regions,
            "number_of_analysis_positions": data.number_of_analysis_positions,
            "number_of_discrete_x_coordinates_in_full_map": data.number_of_discrete_x_coordinates_in_full_map,
            "number_of_discrete_y_coordinates_in_full_map": data.number_of_discrete_y_coordinates_in_full_map,
            "number_of_experimental_variables": data.number_of_experimental_variables,
            "number_of_entries_in_parameter_inclusion_or_exclusion_list": data.number_of_entries_in_parameter_inclusion_or_exclusion_list,
            "number_of_manually_entered_items_in_block": data.number_of_manually_entered_items_in_block,
            "number_of_future_upgrade_experiment_entries": data.number_of_future_upgrade_experiment_entries,
            "number_of_future_upgrade_block_entries": data.number_of_future_upgrade_block_entries,
            "number_of_blocks": data.number_of_blocks,
            "experiment_terminator": data.experiment_terminator,
        })
        self.client.create_table_data(database_id, table_id, data=payload)
        time.sleep(2)
        query = "select id as id from experiments order by id DESC LIMIT 1"
        return self.client.create_subset(database_id, query).result[0]["id"]

    def insert_experiment_variables(self, database_id: int, table_id: int, experiment_id: int, data: VAMASExperiment):
        for i in range(0, len(data.experimental_variable_labels)):
            payload = dict({
                "experiment_id": experiment_id,
                "label": data.experimental_variable_labels[i],
                "unit": data.experimental_variable_units[i]
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_experiment_parameter_inclusion_or_exclusion_prefix_numbers(self, database_id: int, table_id: int,
                                                                          experiment_id: int, data: VAMASExperiment):
        for number in data.parameter_inclusion_or_exclusion_prefix_numbers:
            payload = dict({
                "experiment_id": experiment_id,
                "number": number,
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_experiment_prefix_numbers_of_manually_entered_items(self, database_id: int, table_id: int,
                                                                   experiment_id: int, data: VAMASExperiment):
        for number in data.prefix_numbers_of_manually_entered_items:
            payload = dict({
                "experiment_id": experiment_id,
                "prefix_number": number,
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_experiment_future_upgrade_experiment_entries(self, database_id: int, table_id: int,
                                                            experiment_id: int, data: VAMASExperiment):
        for entry in data.future_upgrade_experiment_entries:
            payload = dict({
                "experiment_id": experiment_id,
                "entry": entry,
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_block_data(self, database_id: int, table_id: int, experiment_id, block: VAMASBlock) -> int:
        sputtering_mode = None
        if block.sputtering_mode == "continuous" or block.sputtering_mode == "cyclic":
            sputtering_mode = block.sputtering_mode
        payload = dict({
            "experiment_id": experiment_id,
            "block_identifier": block.block_identifier,
            "sample_identifier": block.sample_identifier,
            "date": str(block.date.date()),
            "number_of_hours_in_advance_of_greenwich_mean_time": block.number_of_hours_in_advance_of_greenwich_mean_time,
            "number_of_lines_in_block_comment": block.number_of_lines_in_block_comment,
            "comment": block.comment,
            "technique": block.technique,
            "x_coordinate": block.x_coordinate,
            "y_coordinate": block.y_coordinate,
            "analysis_source_label": block.analysis_source_label,
            "sputtering_ion_or_atomic_number": block.sputtering_ion_or_atomic_number,
            "number_of_atoms_in_sputtering_ion_or_atom_particle": block.number_of_atoms_in_sputtering_ion_or_atom_particle,
            "sputtering_ion_of_atom_charge_sign_and_number": block.sputtering_ion_of_atom_charge_sign_and_number,
            "analysis_source_characteristic_energy": block.analysis_source_characteristic_energy,
            "analysis_source_strength": block.analysis_source_strength,
            "analysis_source_beam_width_x": block.analysis_source_beam_width_x,
            "analysis_source_beam_width_y": block.analysis_source_beam_width_y,
            "analysis_width_x": block.analysis_width_x,
            "analysis_width_y": block.analysis_width_y,
            "field_of_view_x": block.field_of_view_x,
            "field_of_view_y": block.field_of_view_y,
            "first_linescan_start_x_coordinate": block.first_linescan_start_x_coordinate,
            "first_linescan_start_y_coordinate": block.first_linescan_start_y_coordinate,
            "first_linescan_finish_x_coordinate": block.first_linescan_finish_x_coordinate,
            "first_linescan_finish_y_coordinate": block.first_linescan_finish_y_coordinate,
            "last_linescan_finish_x_coordinate": block.last_linescan_finish_x_coordinate,
            "last_linescan_finish_y_coordinate": block.last_linescan_finish_y_coordinate,
            "analysis_source_polar_angle_of_incidence": block.analysis_source_polar_angle_of_incidence,
            "analysis_source_azimuth": block.analysis_source_azimuth,
            "analyser_mode": block.analyser_mode,
            "differential_width": block.differential_width,
            "magnification_of_analyser_transfer_lens": block.magnification_of_analyser_transfer_lens,
            "analyser_work_function_or_acceptance_energy_of_atom_or_ion": block.analyser_work_function_or_acceptance_energy_of_atom_or_ion,
            "target_bias": block.target_bias,
            "analyser_axis_take_off_polar_angle": block.analyser_axis_take_off_polar_angle,
            "analyser_axis_take_off_azimuth": block.analyser_axis_take_off_azimuth,
            "species_label": block.species_label,
            "transition_or_charge_state_label": block.transition_or_charge_state_label,
            "charge_of_detected_particle": block.charge_of_detected_particle,
            "abscissa_label": block.abscissa_label,
            "abscissa_units": block.abscissa_units,
            "abscissa_start": block.abscissa_start,
            "abscissa_increment": block.abscissa_increment,
            "number_of_corresponding_variables": block.number_of_corresponding_variables,
            "signal_mode": block.signal_mode,
            "signal_collection_time": block.signal_collection_time,
            "number_of_scans_to_compile_this_block": block.number_of_scans_to_compile_this_block,
            "signal_time_correction": block.signal_time_correction,
            "sputtering_source_energy": block.sputtering_source_energy,
            "sputtering_source_beam_current": block.sputtering_source_beam_crrent,
            "sputtering_source_width_x": block.sputtering_source_width_x,
            "sputtering_source_width_y": block.sputtering_source_width_y,
            "sputtering_source_polar_angle_of_incidence": block.sputtering_source_polar_angle_of_incidence,
            "sputtering_source_azimuth": None,
            "sputtering_mode": sputtering_mode,
            "sample_normal_polar_angle_of_tilt": block.sample_normal_polar_angle_of_tilt,
            "sample_normal_tilt_azimuth": block.sample_normal_tilt_azimuth,
            "sample_rotation_angle": block.sample_rotation_angle,
            "number_of_additional_numerical_parameters": block.number_of_additional_numerical_parameters,
            "number_of_ordinate_values": block.number_of_ordinate_values,
            "analyser_pass_energy_of_retard_ratio_or_mass_resolution": block.analyser_pass_energy_of_retard_ratio_or_mass_resolution,
            "carriage_return": None
        })
        self.client.create_table_data(database_id, table_id, data=payload)
        return self.client.get_table_data_count(database_id, table_id)

    def insert_block_values_of_experimental_variables(self, database_id: int, table_id: int, block_id: str,
                                                      data: VAMASBlock):
        for variable in data.values_of_experimental_variables:
            payload = dict({
                "block_id": block_id,
                "variable": variable,
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_block_corresponding_variables(self, database_id: int, table_id: int, block_id: str,
                                             data: VAMASBlock):
        for i in range(0, data.number_of_corresponding_variables):
            payload = dict({
                "block_id": block_id,
                "label": data.corresponding_variable_labels[i],
                "unit": data.corresponding_variable_units[i],
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_block_additional_numerical_parameters(self, database_id: int, table_id: int, block_id: str,
                                                     data: VAMASBlock):
        for i in range(0, data.number_of_additional_numerical_parameters):
            payload = dict({
                "block_id": block_id,
                "label": data.additional_numerical_parameter_labels[i],
                "unit": data.additional_numerical_parameter_units[i],
                "value": data.additional_numerical_parameter_values[i],
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_block_future_update_block_entries(self, database_id: int, table_id: int, block_id: str,
                                                 data: VAMASBlock):
        for entry in data.future_upgrade_block_entries:
            payload = dict({
                "block_id": block_id,
                "entry": entry,
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_block_min_max_ordinate_values(self, database_id: int, table_id: int, block_id: int, data: VAMASBlock):
        for i in range(0, data.number_of_corresponding_variables):
            payload = dict({
                "block_id": block_id,
                "minimum": data.minimum_ordinate_values[i],
                "maximum": data.maximum_ordinate_values[i],
            })
            self.client.create_table_data(database_id, table_id, data=payload)

    def insert_block_ordinate_values(self, database_id: int, table_id: int, block_id: str, data: VAMASBlock):
        for i in range(0, len(data.ordinate_values)):
            rows = []
            for value in data.ordinate_values[i]:
                payload = dict({
                    "position": i,
                    "block_id": block_id,
                    "value": value
                })
                rows.append(payload)
            pandas.DataFrame(rows, columns=['position', 'block_id', 'value']).to_csv(path_or_buf="./tmp.csv",
                                                                                      index=False)
            self.client.import_table_data(database_id, table_id, separator=",", file_path="./tmp.csv", skip_lines=1,
                                          line_encoding="\\n")
